'use strict';

var _Promise = require('babel-runtime/core-js/promise').default;

var _regeneratorRuntime = require('babel-runtime/regenerator').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _gracefulFs = require('graceful-fs');

var _gracefulFs2 = _interopRequireDefault(_gracefulFs);

var _pngjs = require('pngjs');

var _utilsEnsureDirectory = require('../utils/ensure-directory');

var _utilsEnsureDirectory2 = _interopRequireDefault(_utilsEnsureDirectory);

var DEFAULT_THUMBNAIL_WIDTH = 240;
var DEFAULT_THUMBNAIL_HEIGHT = 130;

function cubicInterpolation(t, t2, t3, f) {
    var C = [0 - 1 * t + 2 * t2 - 1 * t3, 2 + 0 * t - 5 * t2 + 3 * t3, 0 + 1 * t + 4 * t2 - 3 * t3, 0 + 0 * t - 1 * t2 + 1 * t3];

    return 0.5 * (C[0] * f[0] + C[1] * f[1] + C[2] * f[2] + C[3] * f[3]);
}

function getSubpixel(img, y, x, channel) {
    if (y >= img.height || x >= img.width || y < 0 || x < 0) return 0;

    return img.data[y * img.width * 4 + x * 4 + channel];
}

function setSubpixel(img, y, x, channel, color) {
    if (y >= img.height || x >= img.width || y < 0 || x < 0) return;

    if (color > 255) color = 255;else if (color < 0) color = 0;

    img.data[y * img.width * 4 + x * 4 + channel] = color;
}

function scale(src, width, height) {
    var dst = new _pngjs.PNG({ width: width, height: height });

    var cx = src.width / width;
    var cy = src.height / height;

    var interpolated = [0, 0, 0, 0];

    for (var i = 0; i < height; ++i) {
        var targetY = cy * i;
        var sourceY = Math.floor(targetY);

        var dy = targetY - sourceY;
        var dy2 = dy * dy;
        var dy3 = dy2 * dy;

        for (var j = 0; j < width; ++j) {
            var targetX = cx * j;
            var sourceX = Math.floor(targetX);

            var dx = targetX - sourceX;
            var dx2 = dx * dx;
            var dx3 = dx2 * dx;

            for (var k = 0; k < 3; ++k) {
                for (var ii = 0, interpolY = sourceY - 1; ii < 4; ++ii, ++interpolY) {
                    var pixels = [getSubpixel(src, interpolY, sourceX - 1, k), getSubpixel(src, interpolY, sourceX, k), getSubpixel(src, interpolY, sourceX + 1, k), getSubpixel(src, interpolY, sourceX + 2, k)];

                    interpolated[ii] = cubicInterpolation(dx, dx2, dx3, pixels);
                }

                setSubpixel(dst, i, j, k, cubicInterpolation(dy, dy2, dy3, interpolated));
            }

            setSubpixel(dst, i, j, 3, 255);
        }
    }

    return dst;
}

function generate(src, dst, width, height) {
    return new _Promise(function (resolve, reject) {
        var srcFileData = _gracefulFs2.default.readFileSync(src);

        var srcImage = new _pngjs.PNG();

        srcImage.parse(srcFileData, function (e) {
            if (e) reject(e);

            try {
                var dstImage = scale(srcImage, width, height);

                var dstFileData = _pngjs.PNG.sync.write(dstImage);

                _gracefulFs2.default.writeFileSync(dst, dstFileData);

                resolve();
            } catch (e3) {
                reject(e3);
            }
        });
    });
}

function getThumbnailPath(imagePath) {
    var imageName = _path2.default.basename(imagePath);
    var imageDir = _path2.default.dirname(imagePath);

    return _path2.default.join(imageDir, 'thumbnails', imageName);
}

/**
 * Creates a thumbnail image from the specified PNG image file.
 * @function
 * @async
 * @name generateThumbnail
 * @param {string} sourcePath - Specifies the path to the source image in PNG format.
 * @param {string} thumbnailPath - Specifies the path to the resulting thumbnail image.
 *                                 Defaults to '<sourcePathDirectory>/thumbnails/<sourcePathFileName>'
 * @param {number} width - Specifies the width of the thumbnail image, in pixels (default is 240).
 * @param {number} height - Specifies the height of the thumbnail image, in pixels (default is 130).
 */

exports.default = function callee$0$0(sourcePath) {
    var thumbnailPath = arguments.length <= 1 || arguments[1] === undefined ? getThumbnailPath(sourcePath) : arguments[1];
    var width = arguments.length <= 2 || arguments[2] === undefined ? DEFAULT_THUMBNAIL_WIDTH : arguments[2];
    var height = arguments.length <= 3 || arguments[3] === undefined ? DEFAULT_THUMBNAIL_HEIGHT : arguments[3];
    return _regeneratorRuntime.async(function callee$0$0$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                return context$1$0.abrupt('return', (function callee$1$0() {
                    return _regeneratorRuntime.async(function callee$1$0$(context$2$0) {
                        while (1) switch (context$2$0.prev = context$2$0.next) {
                            case 0:
                                context$2$0.next = 2;
                                return _regeneratorRuntime.awrap(_utilsEnsureDirectory2.default(thumbnailPath));

                            case 2:
                                if (context$2$0.sent) {
                                    context$2$0.next = 4;
                                    break;
                                }

                                return context$2$0.abrupt('return');

                            case 4:
                                context$2$0.next = 6;
                                return _regeneratorRuntime.awrap(generate(...[sourcePath, thumbnailPath, width, height]));

                            case 6:
                            case 'end':
                                return context$2$0.stop();
                        }
                    }, null, this);
                })());

            case 1:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this);
};

module.exports = exports.default;