'use strict';

var _regeneratorRuntime = require('babel-runtime/regenerator').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;

var _gracefulFs = require('graceful-fs');

var _gracefulFs2 = _interopRequireDefault(_gracefulFs);

var _path = require('path');

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _promisify = require('./promisify');

var _promisify2 = _interopRequireDefault(_promisify);

var _delay = require('./delay');

var _delay2 = _interopRequireDefault(_delay);

var stat = _promisify2.default(_gracefulFs2.default.stat);
var open = _promisify2.default(_gracefulFs2.default.open);
var close = _promisify2.default(_gracefulFs2.default.close);
var mkdir = _promisify2.default(_mkdirp2.default);

var MAX_RETRY_COUNT = 10;
var RETRY_DELAY = 100;

function createDirectory(directoryPath) {
    return _regeneratorRuntime.async(function createDirectory$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                context$1$0.prev = 0;
                context$1$0.next = 3;
                return _regeneratorRuntime.awrap(mkdir(directoryPath));

            case 3:
                return context$1$0.abrupt('return', true);

            case 6:
                context$1$0.prev = 6;
                context$1$0.t0 = context$1$0['catch'](0);
                return context$1$0.abrupt('return', false);

            case 9:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this, [[0, 6]]);
}

function openFile(fileName) {
    var fd;
    return _regeneratorRuntime.async(function openFile$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                context$1$0.prev = 0;
                context$1$0.next = 3;
                return _regeneratorRuntime.awrap(open(fileName, 'a'));

            case 3:
                fd = context$1$0.sent;
                context$1$0.next = 6;
                return _regeneratorRuntime.awrap(close(fd));

            case 6:
                return context$1$0.abrupt('return', true);

            case 9:
                context$1$0.prev = 9;
                context$1$0.t0 = context$1$0['catch'](0);
                return context$1$0.abrupt('return', false);

            case 12:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this, [[0, 9]]);
}

function ensureFile(fileName) {
    var directoryPath, fileEnsured, i;
    return _regeneratorRuntime.async(function ensureFile$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                directoryPath = _path.dirname(fileName);
                context$1$0.next = 3;
                return _regeneratorRuntime.awrap(createDirectory(directoryPath));

            case 3:
                context$1$0.t0 = context$1$0.sent;

                if (!context$1$0.t0) {
                    context$1$0.next = 8;
                    break;
                }

                context$1$0.next = 7;
                return _regeneratorRuntime.awrap(openFile(fileName));

            case 7:
                context$1$0.t0 = context$1$0.sent;

            case 8:
                fileEnsured = context$1$0.t0;
                i = 0;

            case 10:
                if (!(i < MAX_RETRY_COUNT && !fileEnsured)) {
                    context$1$0.next = 24;
                    break;
                }

                context$1$0.next = 13;
                return _regeneratorRuntime.awrap(_delay2.default(RETRY_DELAY));

            case 13:
                context$1$0.next = 15;
                return _regeneratorRuntime.awrap(createDirectory(directoryPath));

            case 15:
                context$1$0.t1 = context$1$0.sent;

                if (!context$1$0.t1) {
                    context$1$0.next = 20;
                    break;
                }

                context$1$0.next = 19;
                return _regeneratorRuntime.awrap(openFile(fileName));

            case 19:
                context$1$0.t1 = context$1$0.sent;

            case 20:
                fileEnsured = context$1$0.t1;

            case 21:
                i++;
                context$1$0.next = 10;
                break;

            case 24:
                return context$1$0.abrupt('return', fileEnsured);

            case 25:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this);
}

exports.default = function callee$0$0(fileName) {
    var directoryPath, stats;
    return _regeneratorRuntime.async(function callee$0$0$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                directoryPath = _path.dirname(fileName);
                context$1$0.prev = 1;
                context$1$0.next = 4;
                return _regeneratorRuntime.awrap(stat(directoryPath));

            case 4:
                stats = context$1$0.sent;
                return context$1$0.abrupt('return', stats.isDirectory());

            case 8:
                context$1$0.prev = 8;
                context$1$0.t0 = context$1$0['catch'](1);

                if (!(context$1$0.t0.code === 'ENOENT')) {
                    context$1$0.next = 14;
                    break;
                }

                context$1$0.next = 13;
                return _regeneratorRuntime.awrap(ensureFile(fileName));

            case 13:
                return context$1$0.abrupt('return', context$1$0.sent);

            case 14:
                return context$1$0.abrupt('return', false);

            case 15:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this, [[1, 8]]);
};

module.exports = exports.default;